#include <analysis.h>
#include "Attenuator.h"
#include <formatio.h>
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include <stdio.h>
#include <stdlib.h>
#include "Attenuator.h"
#include "Communication.h"

/*********************************************************************
LabWindows CVI software for attenuator.
Version 1.0 developed by ANDREW on 24/10/97 for Kingfisher Int. P/L
Re-developed by Daniel on 12 Dec 2000
Fixed up to compile under Windows 10 Sep 2015 by Malcolm
*********************************************************************/

//efine VERSION "ATTENUATOR CONTROL V2.0.3" 
#define VERSION "ATTENUATOR CONTROL V2.1.0" 

#define DEFAULT_STARTVAL         2.50        
#define DEFAULT_ENDVAL           60.0        
#define DEFAULT_ATTENUATION      2.0
#define DEFAULT_WAVELENGTHPOINT  0
#define DEFAULT_NUMBEROFSTEPS    1        
#define DEFAULT_STEPSIZE         0.05        
#define DEFAULT_TIMEVAL          1.0

#define DATA_INTTYPE             0
#define DATA_DOUBLETYPE          1
#define DATA_STRINGTYPE          2

#define BAUD                     9600
#define TIME_OUT                 10

typedef struct {
  char company[260];
  int numOfWvl;
  int wvl[4];
  int select[3];
  double start[3];
  double end[3];
  double step[3];
  int time[3];
} TIniHeader;

/*************************** global variables ***************************/

int  	   panel; 

TIniHeader fIni;
int  	   fWvlPoint;       
double   fAttenuation;
int      fStepPoint;          // point to current step set
int      fCurrentMaxSteps;
int      fMaxSteps[3];
int      fStepCount;          // Current number of steps 
int      fSelect;
double   fStart;                  
double   fEnd;
double   fStep;               // Current fStep size
int      fTime;               // Current time value
double   fRange;
double   x1, y1, x2, y2;      // Plotting variables
int      fStart_Stop;         // button start/Stop flag
int fInitialised, fPort = 0;

// define prototype
int ReadIni(void);
int WriteIni(void);
int Print_Message(char *s);
int Get_Atten_Param(void);
int Set_Atten_Param(void);
double ABS(double aVal1, double aVal2);
void GetNumberOfSteps(void);
int SearchCOMport(void);

void main ()
{
  SuspendTimerCallbacks();
  ReadIni();
	
  fInitialised = FALSE;
  panel = LoadPanel (0, "Attenuator.uir", PCAL);
  DisplayPanel (panel);
  RunUserInterface();
}

int CVICALLBACK PCAL_Initialise (int panel, int event, void *callbackData, int eventData1, int eventData2)
{
  char buf[256];
  int i, status;
  
  switch (event) {
  	case EVENT_GOT_FOCUS:
	    SetPanelAttribute(panel,ATTR_TITLE,VERSION);
	    Fmt(buf,"%s<%s%s",fIni.company," Attenuator Remote Control"); SetCtrlVal(panel,PCAL_TEXTMSG_2,buf);
	    ClearListCtrl(panel,PCAL_WAVELENGTH);
	  	for (i = 0; i < fIni.numOfWvl; i++) {Fmt(buf,"%s<%d",fIni.wvl[i]); InsertListItem(panel,PCAL_WAVELENGTH,i,buf,i);}
	  	SetCtrlAttribute(panel, PCAL_STOP, ATTR_DIMMED, 1); SetCtrlAttribute(panel, PCAL_CONTINUE, ATTR_DIMMED, 1);
	  	Set_Atten_Param(); GetNumberOfSteps();
    	fStart_Stop  = TRUE; fInitialised = TRUE;
    	
		  if (fPort == 0) if (SearchCOMport()) return FALSE;
   	  DeleteGraphPlot(panel, PCAL_GRAPH, -1, VAL_IMMEDIATE_DRAW);		
      status = GetCtrlVal(panel,PCAL_ATTENUATION,&fAttenuation);
      if (status >= 0) {AttenuatorGroup_setAttenuation(fAttenuation, fPort, TIME_OUT); SetCtrlVal(panel, PCAL_ATTENUATION, fAttenuation);}	
	  	break;
		}
  return 0;
}

int QUIT (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		  if (fPort > 0) {AttenuatorGroup_exit(fPort, TIME_OUT); RS232release(fPort);}
			Get_Atten_Param(); WriteIni(); QuitUserInterface(0);
			break;
	}
	return 0;
}

/**********
Menu Option
***********/

void CVICALLBACK Option (int menuBar, int menuItem, void *callbackData,int panel)
{
  panel = LoadPanel (0, "Attenuator.uir", SERIAL);
  DisplayPanel (panel);

  switch (fIni.numOfWvl) {
    case 1:
      SetCtrlVal(panel,SERIAL_WAVELENGTH1,fIni.wvl[0]);
      break;
    case 2:
      SetCtrlVal(panel,SERIAL_WAVELENGTH1,fIni.wvl[0]); SetCtrlVal(panel,SERIAL_WAVELENGTH2,fIni.wvl[1]);
      break;
    case 3:
      SetCtrlVal(panel,SERIAL_WAVELENGTH1,fIni.wvl[0]); SetCtrlVal(panel,SERIAL_WAVELENGTH2,fIni.wvl[1]);
      SetCtrlVal(panel,SERIAL_WAVELENGTH3,fIni.wvl[2]);
      break;
    case 4:
      SetCtrlVal(panel,SERIAL_WAVELENGTH1,fIni.wvl[0]); SetCtrlVal(panel,SERIAL_WAVELENGTH2,fIni.wvl[1]);
      SetCtrlVal(panel,SERIAL_WAVELENGTH3,fIni.wvl[2]); SetCtrlVal(panel,SERIAL_WAVELENGTH4,fIni.wvl[3]);
      break;
    }
}

int CVICALLBACK Option_apply (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
{
  int i;
  
  switch (event) {
  	case EVENT_COMMIT:
      GetCtrlVal(panel,SERIAL_WAVELENGTH1,&fIni.wvl[0]); GetCtrlVal(panel,SERIAL_WAVELENGTH2,&fIni.wvl[1]);
      GetCtrlVal(panel,SERIAL_WAVELENGTH3,&fIni.wvl[2]); GetCtrlVal(panel,SERIAL_WAVELENGTH4,&fIni.wvl[3]);

      fIni.numOfWvl = 0;
  	  for (i = 0; i < 4; i++) if (fIni.wvl[i] > 0) fIni.numOfWvl++;
  	  DiscardPanel (panel);
	    break;
	  }
  return 0;
}

int CVICALLBACK Option_cancel (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
{
  switch (event) {
	  case EVENT_COMMIT:
  	  DiscardPanel (panel);
	    break;
	  }
  return 0;
}

/*********************************************************
		User interface callback functions
*********************************************************/
int CVICALLBACK Main_wavelength (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
   switch (event) {
      case EVENT_COMMIT:
         if (fPort == 0) if (SearchCOMport()) return FALSE;
         GetCtrlIndex(panel,PCAL_WAVELENGTH,&fWvlPoint);
         AttenuatorGroup_selectLambda(fWvlPoint, fPort, TIME_OUT); AttenuatorGroup_setLambda(fPort, TIME_OUT);
         break;
      }
	return 0;
}

int Main_goto (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
{
   int status;

   switch (event) {
      case EVENT_COMMIT:
         if (fPort == 0) if (SearchCOMport()) return FALSE;
         DeleteGraphPlot(panel, PCAL_GRAPH, -1, VAL_IMMEDIATE_DRAW);		
         status = GetCtrlVal(panel,PCAL_ATTENUATION,&fAttenuation);
         if (status >= 0) {AttenuatorGroup_setAttenuation(fAttenuation, fPort, TIME_OUT); SetCtrlVal(panel, PCAL_ATTENUATION, fAttenuation);}	
         break;
   }
   return 0;
}

int Main_start (int panel,int control,int event,void *callbackData, int eventData1, int eventData2)
{
  int     i, totalTime, status = FALSE;

	switch (event) {
		case EVENT_COMMIT:
		  if (fPort == 0) {if (SearchCOMport()) status = TRUE;}
		  else status = TRUE;

			if (status) {
      	DeleteGraphPlot(panel, PCAL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
	  	  SetCtrlAttribute(panel, PCAL_GOTO, ATTR_DIMMED, 1); SetCtrlAttribute(panel, PCAL_START, ATTR_DIMMED, 1);
  		  SetCtrlAttribute(panel, PCAL_STOP, ATTR_DIMMED, 0); SetCtrlAttribute(panel, PCAL_CONTINUE, ATTR_DIMMED, 1);
  		  Get_Atten_Param();
		  
	  	  fStepPoint       = -1;
  	  	fTime            = 1;
	  	  fStepCount       = 0;
		  	fCurrentMaxSteps = 0;
  		  x2             	 = fTime;
    		x1 = y1 = y2 = 0;

	    	totalTime = 0;
		    for (i = 0; i < 3; i++) if (fIni.select[i]) totalTime = totalTime + (fMaxSteps[i] +1) * fIni.time[i];
		    
    		PlotPoint(panel, PCAL_GRAPH, totalTime, 0, VAL_SIMPLE_DOT, VAL_BLACK);
	    	ResetTimer(panel, PCAL_TIMER); ResumeTimerCallbacks();
  	    }
  		break;
	}
	return 0;
}

int TimerCallback(int panel,int control,int event,void *callbackData,int eventData1,int eventData2)
{
  int     i, vStatus, vStepPoint;
  char    buf[260];
  
	switch (event) {
		case EVENT_TIMER_TICK:
			ProcessSystemEvents();
			fTime--;
		  if (fTime == 0) {
		    if (fStepCount >= fCurrentMaxSteps) {
		      vStepPoint = fStepPoint;
    		  for (i = vStepPoint+1; i< 3; i++) if (fIni.select[i]) {fStepPoint = i; break;}   
		      if (fStepPoint == vStepPoint) {
    		    //STOP(panel,control,EVENT_COMMIT,callbackData,0,0);
    		    SetCtrlAttribute(panel, PCAL_CONTINUE, ATTR_DIMMED, 1);
		        break;  
		        }																					
		      fSelect          = fIni.select[fStepPoint];
    		  fStart           = fIni.start[fStepPoint];
		      fEnd             = fIni.end[fStepPoint];
    		  fStep            = fIni.step[fStepPoint];
		      fTime            = fIni.time[fStepPoint];
		      fCurrentMaxSteps = fMaxSteps[fStepPoint];
    		  fStepCount       = 1;
		      if (y2 == fStart) fStepCount++;    		  
    		  y2            = fStart;    		  
  		  } else fStepCount++;
		  
		    Beep(); 
		    fTime = fIni.time[fStepPoint];
		    
    		x1 = x2;
    		y1 = y2;
		  	x2 = x2 + fTime;

        if (fStepCount != 1) {
  			  if (fStart < fEnd) {y2 = y2 + fStep; if (y2 > fEnd) y2 = fEnd;}
  		  	else {y2 = y2 - fStep; if (y2 < fEnd) y2 = fEnd;}
	  			}  

	  	  SetCtrlVal(panel, PCAL_ATTENUATION, y2); AttenuatorGroup_setAttenuation(y2, fPort, TIME_OUT);	  	  

   	  	PlotLine(panel, PCAL_GRAPH, x1, y1, x1, y2, VAL_GREEN);
  			for (i=x1; i<x2; i++) {PlotLine(panel, PCAL_GRAPH, x1, y2, i+(x2-x1)/10, y2, VAL_GREEN);}
   	  	PlotLine(panel, PCAL_GRAPH, x1, y2, x2, y2, VAL_GREEN);
		    
  	  } else break;

		  Fmt(buf,"%s<%s%d%s%d","step ", fStepCount, " of ",fMaxSteps[fStepPoint]);  
		  switch(fStepPoint) {
		    case 0: SetCtrlVal(panel,PCAL_Text1,buf); break;
		    case 1: SetCtrlVal(panel,PCAL_Text2,buf); break;
		    case 2: SetCtrlVal(panel,PCAL_Text3,buf); break;
		    }

   	  break;
	}
	return 0;
}

int Main_stop(int panel,int control,int event,void *callbackData,int eventData1,int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		  SuspendTimerCallbacks();
		  SetCtrlAttribute(panel, PCAL_GOTO, ATTR_DIMMED, 0);		
		  SetCtrlAttribute(panel, PCAL_START, ATTR_DIMMED, 0);
		  SetCtrlAttribute(panel, PCAL_STOP, ATTR_DIMMED, 1);
		  SetCtrlAttribute(panel, PCAL_CONTINUE, ATTR_DIMMED, 0);
			break;
	}
	return 0;
}

int CVICALLBACK Main_continue(int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
  		ResumeTimerCallbacks();
		  SetCtrlAttribute(panel, PCAL_GOTO, ATTR_DIMMED, 1);		
		  SetCtrlAttribute(panel, PCAL_START, ATTR_DIMMED, 1);
		  SetCtrlAttribute(panel, PCAL_STOP, ATTR_DIMMED, 0);
		  SetCtrlAttribute(panel, PCAL_CONTINUE, ATTR_DIMMED, 1);
			break;
		}
	return 0;
}

int CVICALLBACK Steps_Changed(int panel,int control,int event,void *callbackData,int eventData1,int eventData2)
{
  if (fInitialised) {Get_Atten_Param(); GetNumberOfSteps();}  
	return 0;
}

/*****************
 Private functions
******************/

int Get_Atten_Param(void)
{
  GetCtrlVal(panel,PCAL_Select1,&fIni.select[0]);
  GetCtrlVal(panel,PCAL_Start1,&fIni.start[0]);
  GetCtrlVal(panel,PCAL_End1,&fIni.end[0]);		
  GetCtrlVal(panel,PCAL_Step1,&fIni.step[0]);		
  GetCtrlVal(panel,PCAL_Time1,&fIni.time[0]);
  
  GetCtrlVal(panel,PCAL_Select2,&fIni.select[1]);
  GetCtrlVal(panel,PCAL_Start2,&fIni.start[1]);
  GetCtrlVal(panel,PCAL_End2,&fIni.end[1]);		
  GetCtrlVal(panel,PCAL_Step2,&fIni.step[1]);		
  GetCtrlVal(panel,PCAL_Time2,&fIni.time[1]);
  
  GetCtrlVal(panel,PCAL_Select3,&fIni.select[2]);
  GetCtrlVal(panel,PCAL_Start3,&fIni.start[2]);
  GetCtrlVal(panel,PCAL_End3,&fIni.end[2]);		
  GetCtrlVal(panel,PCAL_Step3,&fIni.step[2]);		
  GetCtrlVal(panel,PCAL_Time3,&fIni.time[2]);

  GetCtrlVal(panel,PCAL_WAVELENGTH,&fWvlPoint);
  GetCtrlVal(panel,PCAL_ATTENUATION,&fAttenuation);
  
  return 0;
}

int Set_Atten_Param(void)
{
  char buf[260];
  
  Fmt(buf,"%s<%d",fIni.wvl[0]);  

  SetCtrlVal(panel,PCAL_Select1,fIni.select[0]);
  SetCtrlVal(panel,PCAL_Start1,fIni.start[0]);
  SetCtrlVal(panel,PCAL_End1,fIni.end[0]);  
  SetCtrlVal(panel,PCAL_Step1,fIni.step[0]);
  SetCtrlVal(panel,PCAL_Time1,fIni.time[0]);
  
  SetCtrlVal(panel,PCAL_Select2,fIni.select[1]);
  SetCtrlVal(panel,PCAL_Start2,fIni.start[1]);
  SetCtrlVal(panel,PCAL_End2,fIni.end[1]);  
  SetCtrlVal(panel,PCAL_Step2,fIni.step[1]);
  SetCtrlVal(panel,PCAL_Time2,fIni.time[1]);
  
  SetCtrlVal(panel,PCAL_Select3,fIni.select[2]);
  SetCtrlVal(panel,PCAL_Start3,fIni.start[2]);
  SetCtrlVal(panel,PCAL_End3,fIni.end[2]);  
  SetCtrlVal(panel,PCAL_Step3,fIni.step[2]);
  SetCtrlVal(panel,PCAL_Time3,fIni.time[2]);
										
  return 0;
}

void GetNumberOfSteps(void)
{
  char    buf[260];
  int     i;
  
	for (i = 0; i < 3; i++) {
	  fMaxSteps[i] = ABS(fIni.start[i], fIni.end[i])/fIni.step[i];
	  if (ABS(fIni.start[i],fIni.end[i]) > (fMaxSteps[i]*fIni.step[i])) fMaxSteps[i]++;
	  fMaxSteps[i]++;
	  }
  
  Fmt(buf,"%s<%s%d%s","Total ",fMaxSteps[0], " steps");  
  SetCtrlVal(panel,PCAL_Text1,buf);
  Fmt(buf,"%s<%s%d%s","Total ",fMaxSteps[1], " steps");  
  SetCtrlVal(panel,PCAL_Text2,buf);
  Fmt(buf,"%s<%s%d%s","Total ",fMaxSteps[2], " steps");  
  SetCtrlVal(panel,PCAL_Text3,buf);
}

int Print_Message(s)
char *s;
{
  char buf[10];

  SetCtrlVal(panel,PCAL_PC_Message,s);
  SetCtrlVal(panel, PCAL_PC_Message, "\n");
  
  return 0;
}

int ReadIni(void)
{
  IniText iniText;
  char pathName[MAX_PATHNAME_LEN], buf[MAX_PATHNAME_LEN], *str;
  int i;

  GetProjectDir(buf);
  MakePathname(buf, "Attenuator.ini", pathName);

  iniText = Ini_New (TRUE);
	Ini_ReadFromFile (iniText, pathName);

	Ini_GetStringCopy (iniText, "Header", "company", &str); if (str != NULL) strcpy(fIni.company,str);
  Ini_GetInt(iniText,"Header","Number of wavelength",&fIni.numOfWvl);
  for (i = 0; i < fIni.numOfWvl; i++) {Fmt(buf,"%s<Wvl %d",i+1); Ini_GetInt(iniText,"Header",buf,&fIni.wvl[i]);}

	for (i = 0; i < 3; i++) {
	  Fmt(buf,"%s<select %d",i+1); Ini_GetInt (iniText,"step settings",buf,&fIni.select[i]);
	  Fmt(buf,"%s<start %d",i+1); Ini_GetDouble (iniText,"step settings",buf,&fIni.start[i]);
	  Fmt(buf,"%s<end %d",i+1); Ini_GetDouble (iniText,"step settings",buf,&fIni.end[i]);
	  Fmt(buf,"%s<step %d",i+1); Ini_GetDouble (iniText,"step settings",buf,&fIni.step[i]);
	  Fmt(buf,"%s<time %d",i+1); Ini_GetInt (iniText,"step settings",buf,&fIni.time[i]);	  	  
	  }
  Ini_Dispose (iniText); 
  
  return TRUE;
}

int WriteIni(void)
{
  IniText iniText;
  char pathName[MAX_PATHNAME_LEN], buf[MAX_PATHNAME_LEN];
	int i, n = 0;
	
  GetProjectDir(buf);
  MakePathname(buf, "Attenuator.ini", pathName);			
  iniText = Ini_New (TRUE);

  Ini_PutString (iniText, "Header","company", fIni.company);  
  for (i = 0; i < fIni.numOfWvl; i++) {
    if (fIni.wvl[i] > 0) {Fmt(buf,"%s<Wvl %d",i+1); Ini_PutInt(iniText,"Header",buf,fIni.wvl[i]);}
    else n++;
    }
  fIni.numOfWvl = fIni.numOfWvl - n; Ini_PutInt (iniText,"Header", "Number of wavelength", fIni.numOfWvl);
  
	for (i = 0; i < 3; i++) {
	  Fmt(buf,"%s<select %d",i+1); Ini_PutInt (iniText,"step settings",buf,fIni.select[i]);
	  Fmt(buf,"%s<start %d",i+1); Ini_PutDouble (iniText,"step settings",buf,fIni.start[i]);
	  Fmt(buf,"%s<end %d",i+1); Ini_PutDouble (iniText,"step settings",buf,fIni.end[i]);
	  Fmt(buf,"%s<step %d",i+1); Ini_PutDouble (iniText,"step settings",buf,fIni.step[i]);
	  Fmt(buf,"%s<time %d",i+1); Ini_PutDouble (iniText,"step settings",buf,fIni.time[i]);	  	  
	  }
  Ini_WriteToFile (iniText, pathName);
  Ini_Dispose (iniText); 

  return TRUE;
}

double ABS(double aVal1, double aVal2)
{
  double vVal;
  
  vVal = aVal1 - aVal2;
  if (vVal < 0) vVal = -vVal;
  
  return vVal;
}

int SearchCOMport(void)
{
   int i;
   char buf[260];
  
   for (i = 1; i < 32; i++) {
// for (i = 1; i < 5; i++) {
      Fmt(buf,"%s<Searching for Attenuator at COM%d . . .",i); Print_Message(buf);
      if (RS232initialise(i, BAUD, TIME_OUT) == 0)  
         if (AttenuatorGroup_entry(i, TIME_OUT)) {fPort = i; break;}
      ProcessSystemEvents();  
   }  

   if (fPort == 0) {
      Fmt(buf,"%s<There is no attenuator connected!"); Print_Message(buf);
      return FALSE;
   }
   else {
      Fmt(buf,"%s<Found an attenuator is connected at COM%d",fPort); Print_Message(buf);
      AttenuatorGroup_selectLambda(fWvlPoint, fPort, TIME_OUT);
      AttenuatorGroup_setLambda(fPort, TIME_OUT);
      return TRUE;
   }
}













